BC_Conditions = {}

--===================================================================================================================--

--Testing Condition Functions

--===================================================================================================================--

function BC_Conditions.Verify(thisCry, actionName)
	BC_Core.Message("BC_Conditions.Verify(" .. thisCry.text .. "...)", 2)
	if(thisCry.condition) then
		for i, v in ipairs(thisCry.condition) do
			if(BC_Conditions.Evaluate(v, actionName)~= true) then
				BC_Core.Message("Condition Not Verified", 3)
				return 0;
			end
		end
		BC_Core.Message("Condition Verified", 3)
	end
	for i, v in ipairs(BC_Vars.Condition.Variables) do
		if(string.find(thisCry.text, v)) then
			if(v == "$nm" and #(BC_Core.stored.target) > 0) then
				BC_Core.Message("Pulling $nm from event call", 3)
			else
				func = getglobal(BC_Vars.Condition.Functions[i])
				if(v == "$pn") then
					target = "pet"
				else
					target = "target"
				end
				_, retval = pcall(func,target)
				BC_Core.Message("Calling " .. BC_Vars.Condition.Functions[i] .. "(" .. target .. ") returned " .. tostring(retval), 3)
				if(retval == nil) then
					BC_Core.Message("Cannot evaluate text variable", 3)
					return 0
				end
			end
		end
	end
	loc = string.find(thisCry.text, "$e")
	while(loc) do
		index = tonumber(strsub(thisCry.text,loc+2,loc+2))
		if(BC_Core.eventVars[index]) then
			loc = strfind(thisCry.text, "$e", loc + 1)
		else
			return 0
		end
	end
	return 1
end

function BC_Conditions.Evaluate(condition, actionName)
	BC_Core.Message("BC_Conditions.Evaluate(...)", 2)

	if(condition.target == "event") then
		condition.target = tonumber(string.sub(condition.variable,3,3))
		condition.variable = "$e#"
	end

	if(condition.variable~=1 and condition.target==3 and #(BC_Core.stored.target) > 0) then
		if(UnitName("target") ~= tostring(BC_Core.stored.target[actionName])) then
			return false
		end
	end
	if(condition.variable==1 and condition.target==3 and #(BC_Core.stored.target) > 0) then
		retval = tostring(BC_Core.stored.target[actionName])
	else
		func = getglobal(BC_Vars.Condition.Functions[condition.variable])
		_, retval = pcall(func, BC_Vars.Condition.Targets[condition.target])
	end
	if(BC_Vars.Condition.Types[condition.variable]==2) then
		condition.value = tonumber(condition.value)
	end
	if(BC_Vars.Condition.Types[condition.variable]==3 and retval == nil) then
		retval = "false"
	end
	if(condition.test == 1) then
		if(retval == condition.value) then return true
		else return false; end
	elseif(condition.test == 2) then
		if(retval < condition.value) then return true
		else return false; end
	elseif(condition.test == 3) then
		if(retval <= condition.value) then return true
		else return false; end
	elseif(condition.test == 4) then
		if(retval > condition.value) then return true
		else return false; end;
	elseif(condition.test == 5) then
		if(retval >= condition.value) then return true
		else return false; end
	elseif(condition.test == 6) then
		if(retval ~= condition.value) then return true
		else return false; end
	elseif(condition.test == 7) then
		if(strfind(retval, condition.value)) then return true
		else return false; end
	elseif(condition.test == 8) then
		if(strfind(strlower(retval), strlower(condition.value))) then return true
		else return false; end
	else
		return false;
	end
end

--===================================================================================================================--

--Unit Condition Functions

--===================================================================================================================--

function BC_Conditions.UnitHealthPercent(target)
	if(target) then
		healthMax = UnitHealthMax(target)
		healthCur = UnitHealth(target)
		return tonumber((healthCur / healthMax) * 100)	
	else
		return nil
	end
end

function BC_Conditions.UnitManaPercent(target)
	if(target) then
		manaMax = UnitManaMax(target)
		manaCur = UnitMana(target)
		if(manaCur == 0) then
			return 0
		end
		return tonumber((manaCur / manaMax) * 100)	
	else
		return nil
	end
end

function BC_Conditions.UnitGuildName(target)
	name, _, _ = GetGuildInfo(target)
	return name
end

function BC_Conditions.UnitReaction(target)
	return UnitReaction(target, "player")
end

function BC_Conditions.UnitInParty(target)
	if(target == "player") then
		if(GetNumPartyMembers() > 0) then
			return true
		end
	elseif(UnitInParty(target) == 1) then
		return true
	end
	return false
end

function BC_Conditions.UnitInRaid(target)
	if(target == "player") then
		if(GetNumRaidMembers() > 0) then
			return true
		end
	elseif(UnitInRaid(target) == 1) then
		return true
	end
	return false
end

function BC_Conditions.UnitItemMH()
	itemLink = GetInventoryItemLink("player",16)
	if(itemLink) then
		itemName,_,_,_,_,_,_,_,_,_ = GetItemInfo(itemLink)
		return itemName
	end
	return nil
end

function BC_Conditions.UnitItemOH()
	itemLink = GetInventoryItemLink("player",17)
	if(itemLink) then
		itemName,_,_,_,_,_,_,_,_,_ = GetItemInfo(itemLink)
		return itemName
	end
	return nil
end

function BC_Conditions.UnitItemRW()
	itemLink = GetInventoryItemLink("player",18)
	if(itemLink) then
		itemName,_,_,_,_,_,_,_,_,_ = GetItemInfo(itemLink)
		return itemName
	end
	return nil
end