BC_Data = {}
BC_Data.custom = {}
BC_Data.events = {}
BC_Data.cries = {}

BC_Core = {}
	
BC_Core.stored = {
	events = {},
	cry = {},
	target = {},
	lastCast = nil,
	lastCry = nil,
	events = nil,
	cooldowns = {},
	}

--===================================================================================================================--

-- Debug Functions

--===================================================================================================================--

function BC_Core.Message(message, debugLevel)
	if(debugLevel) then
		if(BC_Vars.DebugLevel >= debugLevel) then
			thisColor = { 
				[1] = "|CFFFFFFFF",
				[2] = "|CFFA0A0A0",
				[3] = "|CFFFF8080 - ",
				[4] = "|CFF80FF80 - - ",
				[5] = "|CFF8080FF - - - ",
			}
			message = thisColor[debugLevel] .. tostring(message) .. "|r"
			DEFAULT_CHAT_FRAME:AddMessage(tostring(message))
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage(tostring(message))
	end
end

function BC_OutputTable(value, padding)
	if(not padding) then padding = ""; end
	for k, v in pairs(value) do
		if(type(v)=="table") then 
			BC_Core.Message(padding .. k .. ":")
			BC_OutputTable(v, padding .. "- ")
		else
			message = padding .. tostring(k) .. " = '" .. tostring(v) .. "'"
			BC_Core.Message(message)
		end
	end
end

--===================================================================================================================--

-- Initialization Functions

--===================================================================================================================--
function BC_CoreFrame_OnLoad()
	BC_Core.Message(BC_UI.AddOnName .. " Core Loaded.")
	BC_Vars.DebugLevel = 0

	if(SLASH_BATTLECRY1) then
		BC_Core.Message("Conflicting versions of BattleCry are loaded, errors may occur.\nPlease disable old version in Addon selection screen. If you need to update, use: /battlecry update")
		SLASH_BATTLECRY1 = nil
		SLASH_BATTLECRY2 = nil
		SlashCmdList["BATTLECRY"] = nil
	end
	SLASH_BATTLECRY1 = "/bc";
	SLASH_BATTLECRY2 = "/battlecry";
	SlashCmdList["BATTLECRY"] = BC_Core.SlashHandler;

	hooksecurefunc("CastPetAction", BC_Core.CastPetAction)
	this:RegisterEvent("VARIABLES_LOADED")
end

function BC_CoreFrame_OnEvent()
	if(BC_Vars.DebugLevel > 10) then
		msg = ""
		i = 1;
		while(getglobal("arg" .. i)) do
			msg = msg .. "[" .. i .. ":" .. getglobal("arg" .. i) .. "] "
			i = i + 1
		end
		BC_Core.Message(msg, 4);
	end

	if(event == "VARIABLES_LOADED") then
		BC_Core.Message(event, 1)
		BC_Events.Initialize()
		BC_Events.RegisterEvents()
		this:RegisterEvent("UNIT_SPELLCAST_SENT")
		this:RegisterEvent("UNIT_SPELLCAST_START")
		this:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
		this:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
		this:RegisterEvent("COMBAT_LOG_EVENT")

		this:UnregisterEvent("VARIABLES_LOADED")
	elseif(event == "UNIT_SPELLCAST_SENT" and arg1 == "player") then
		BC_Core.Message(event, 1)
		if(arg4 and arg4 ~= "") then
			BC_Core.Message("Storing target " .. arg4, 4)
			BC_Core.stored.target[arg2] = arg4
		else
			BC_Core.Message("No target for action", 4)
			BC_Core.stored.target[arg2] = nil
		end
		BC_Core.DetermineCry(arg2)
	elseif(event == "UNIT_SPELLCAST_START") then
		BC_Core.Message(event, 1)
		if(arg1 == "player") then
			BC_Core.ExecuteStoredStartCry(arg2)
			if(BC_Vars.echo) then
				BC_Core.Message("Cast Start: [" .. arg2 .. "]")
			end
		end
	elseif(event == "UNIT_SPELLCAST_SUCCEEDED" and arg1 == "player") then
		BC_Core.Message(event, 1)
		BC_Core.ExecuteStoredCry(arg2)
		if(BC_Vars.echo) then
			BC_Core.Message("Cast Completed: [" .. arg2 .. "]")
		end
	elseif(event == "UNIT_SPELLCAST_INTERRUPTED" and arg1 == "player") then
		BC_Core.Message(event, 1)
		if(arg2) then
			BC_Core.ExecuteInterruptedCry(arg2)
			if(BC_Vars.echo) then
				BC_Core.Message("Cast Interrupted: [" .. arg2 .. "]")
			end
		end
	elseif(event == "COMBAT_LOG_EVENT") then
		if(arg3 == UnitGUID("pet")) then
			BC_Core.Message("Pet casting spell.", 3)
			if(arg2 == "SPELL_CAST_SUCCESS" or arg2 == "SPELL_CAST_START") then
				BC_Core.ExecuteStoredCry("Pet " .. arg10)
				if(BC_Vars.echo) then
					BC_Core.Message("Cast Start: [Pet " .. arg10 .. "]")
				end
			end
		end
	else
		BC_Core.Message(event, 1)
		BC_Events.Process(event)
	end
end

function BC_Core.SlashHandler(msg)
	BC_Core.Message("BC_Core.SlashHandler(" .. msg .. ")", 2);
	if(strsub(msg,1,5) == "debug") then
		local level = tonumber(strsub(msg,7)) or BC_Vars.DebugLevel
		BC_Core.Message("Setting BattleCry Debug Level to: " .. level)
		BC_Vars.DebugLevel = level
	elseif(strsub(msg,1,6) == "update") then
		BC_Core.Message("Attempting to update from previous version of BattleCry.")
		BC_Core.Update()
	elseif(strsub(msg,1,4) == "echo") then
		BC_Vars.echo = not BC_Vars.echo
		BC_Core.Message("Echoing set to " .. tostring(BC_Vars.echo) .. ".")
	elseif(strsub(msg,1,4) == "cry ") then
		actionName = strsub(msg,5)
		BC_Core.Message("|" .. actionName .. "|", 3)
		BC_Core.DetermineCry(actionName)
		BC_Core.ExecuteStoredCry(actionName)
	elseif(strsub(msg,1,6) == "editor") then
		if(not IsAddOnLoaded("BattleCry_Editor")) then
			if(BC_Core.LoadEditor()) then
				BC_EditorFrame:Show()
			end
		else
			BC_EditorFrame:Show()
		end
	else
		BC_Core.Message(BC_Vars.Title .. ":")
		BC_Core.Message("|CFF80FF80/battlecry|r or |CFF80FF80/bc|r for commands")
		BC_Core.Message("|CFF80FF80/battlecry echo|r to print out action names as you do them")
		BC_Core.Message("|CFF80FF80/battlecry cry Action Name|r to execute a random cry for the entered action")
		BC_Core.Message("|CFF80FF80/battlecry editor|r to open the editor")
	end
end

function BC_Core.LoadEditor()
	BC_Core.Message("BC_Core.LoadEditor()", 2)
	local addonLoaded, reason = LoadAddOn("BattleCry_Editor")

	if not addonLoaded then
		if(reason == "DISABLED") then
			BC_Core.Message("You must enable the BattleCry Editor before you can edit cries.")
		else
			BC_Core.Message("Cannot load BattleCry Editor: " .. reason)
		end
		return false
	end
	return true
end

function BC_Core.Update()
	BC_Core.Message("BC_Core.Update()", 2)
	if(BC_Saved) then
		if(BC_Saved.actions) then
			BC_Core.Message("Updating Custom Actions...")
			msg = ". "
			for k, v in pairs(BC_Saved.actions) do
				msg = msg .. ". "
				BC_Core.Message("Updating Custom Action [" .. v .. "]", 3)
				if(BC_Data.custom) then
					add = true
					for i, u in ipairs(BC_Data.custom) do
						if(u == v) then
							add = false
							BC_Core.Message(v .. " is already in Custom Action list, ignoring.", 4)
						end
					end
					if add then
						BC_Core.Message(v .. " is not in Custom Action list, adding.", 4)
						table.insert(BC_Data.custom, v)
					end
				else
					BC_Core.Message("First custom action, creating table.",4)
					BC_Data.custom = { v }
				end
			end
			BC_Core.Message(msg .. " Complete!")
		end
		if(BC_Saved.cries) then
			BC_Core.Message("Updating Saved Cries...")
			msg = ". "
			for k, v in pairs(BC_Saved.cries) do
				msg = msg .. ". "
				if(#(v.cry) > 0) then
					BC_Core.Message("Updating Action [" .. k .. "]", 3)
					for i, cry in ipairs(v.cry) do
						msg = msg .. ". "
						oldCry = {
							name = cry.name,
							text = cry.text,
							mult = cry.frequency,
							language = cry.language,
							mode = cry.mode,
							condition = {},
							}
						if(cry.castCode == 2) then
							oldCry.start = true
						elseif(cry.castCode == 3) then
							oldCry.interrupt = true
						end
						if(cry.always == 1) then
							oldCry.always = true
						end
						if(cry.exclusive == 1) then
							oldCry.exclusive = true
						end
						if(not BC_Data.cries[k]) then
							BC_Core.Message("Creating action " .. k, 4)
							BC_Data.cries[k] = {
								cries = {},
								freq = v.frequency,
								cooldown = v.cooldown,
							}
						end
						for j=1, #(BC_Data.cries[k].cries) do
							if(BC_Data.cries[k].cries[j].name == oldCry.name) then
								--BC_Core.Message(BC_Data.cries[k].cries[j].name, 5)
								oldCry.name = oldCry.name .. " (old)"
							end
						end
						BC_Core.Message("Saving Cry [" .. oldCry.name .. "]", 4)
						table.insert(BC_Data.cries[k].cries, oldCry)
					end
				end
			end
			BC_Core.Message(msg .. " Complete!")
		end
		if(BC_Saved.events) then
			BC_Core.Message("Updating Saved Event Watches...")
			msg = ". "
			for k, v in pairs(BC_Saved.events) do
				msg = msg .. ". "
				BC_Core.Message("Updating Event [" .. k .. "] " .. v.event, 3)
				event = {	["name"] = k,	}
				if v.args then
					event.args = {}
					for i, arg in ipairs(v.args) do
						BC_Core.Message("Arg " .. i .. ": " .. arg, 4)
						if arg then
							table.insert(event.args, arg)
						end
					end
				end
				if(BC_Data.events[v.event]) then
					for i, u in pairs(BC_Data.events[v.event]) do
						if(u.name == k) then
							event["name"] = event["name"] .. " (old)"
						end
					end
					table.insert(BC_Data.events[v.event], event)
				else
					BC_Data.events[v.event] = {event,}
				end
			end
			BC_Core.Message(msg .. " Complete!")
		end
		if(BC_Saved.chains) then
			BC_Core.Message("Updating Cry Chains...")
			msg = ". "
			for k, v in pairs(BC_Saved.chains) do
				msg = msg .. ". "
				BC_Core.Message("Updating Chain [" .. v.name .. "]", 3)
				if BC_Data.cries[v.actionName1] and BC_Data.cries[v.actionName2] then
					if #(BC_Data.cries[v.actionName1].cries) > 0 and #(BC_Data.cries[v.actionName2].cries) > 0 then
						BC_Core.Message("Looking for " .. v.cry2 .. " in " .. v.actionName2, 4)
						found = 0
						for i, u in ipairs(BC_Data.cries[v.actionName1].cries) do
							if u.name == v.cry1 then
								found = i
							end
						end
						if found > 0 then
							BC_Core.Message("Found at " .. found, 3)
							if not BC_Data.cries[v.actionName1].cries[found].chains then
								BC_Core.Message("Creating table for chains", 5)
								BC_Data.cries[v.actionName1].cries[found].chain = {}
							end
							if not BC_Data.cries[v.actionName1].cries[found].chain[v.actionName2] then
								BC_Data.cries[v.actionName1].cries[found].chain[v.actionName2] = {}
								BC_Core.Message("Creating table for " .. v.actionName2, 5)
							end
							table.insert(BC_Data.cries[v.actionName1].cries[found].chain[v.actionName2], found)
						end
					end
				end
			end
			BC_Core.Message(msg .. " Complete!")
		end
	end
end
--===================================================================================================================--

-- Establish Cry Functions

--===================================================================================================================--
function BC_Core.DetermineCry(actionName)
	BC_Core.Message("BC_Core.DetermineCry(" .. actionName .. ")", 2)
	if(BC_Core.OnCooldown(actionName)) then
		BC_Core.Message(actionName .. " is still on cooldown", 3)
		return
	end
	local cryNumber = nil
	local cryList = {}
	if(BC_Data.cries[actionName]) then
		local chance = random(1,100)
		BC_Core.Message(chance .. " against frequency " .. BC_Data.cries[actionName].freq, 3)
		cryList = BC_Core.GetCryList(actionName)
		cryList = BC_Core.PruneCryList(actionName, cryList, "interrupt", false)
		if(#(cryList)>0) then
			if(chance <= BC_Data.cries[actionName].freq) then
				cryList = BC_Core.GetCryListMultiplied(actionName, cryList)
				cryNumber = cryList[random(1,#(cryList))]
			else
				cryList = BC_Core.GetCryListMultiplied(actionName, BC_Core.PruneCryList(actionName, cryList, "always", true))
			end
			if(#(cryList)>0) then
				cryNumber = cryList[random(1,#(cryList))]
				BC_Core.stored.cry[actionName] = BC_Core.FormatCry(BC_Data.cries[actionName].cries[cryNumber], actionName)
				BC_Core.stored.lastCast = actionName
				BC_Core.stored.lastCry = cryNumber
			else
				BC_Core.stored.cry[actionName] = nil
				BC_Core.stored.lastCast = nil
				BC_Core.stored.lastCry = nil
			end
		end
	end
end

function BC_Core.GetCryText(actionName, cryNumber)
	BC_Core.Message("BC_Core.GetCryText(" .. actionName .. ", " .. cryNumber ..")", 2)
	if(cryNumber > 0) then
		return BC_Data.cries[actionName].cries[cryNumber].text
	else
		return ""
	end
end

function BC_Core.GetCryListMultiplied(actionName, inputList)
	BC_Core.Message("BC_Core.GetCryListMultiplied(...)", 2)
	local outputList = {}
	local message = ""
	for k, v in pairs(inputList) do
		message = message .. " " .. v
		if(BC_Data.cries[actionName].cries[v].mult) then
			message = message .. "x" .. BC_Data.cries[actionName].cries[v].mult
			for i=1, BC_Data.cries[actionName].cries[v].mult, 1 do
				table.insert(outputList, v)
			end
		else
			table.insert(outputList, v)
		end
	end
	BC_Core.Message("Multiplied Cry List: " .. message, 4)
	return outputList
end

function BC_Core.PruneCryList(actionName, inputList, criteria, method)
	BC_Core.Message("BC_Core.PruneCryList(...)", 2)
	local outputList = {}
	local message = ""
	for k, v in pairs(inputList) do
		if(method) then
			if(BC_Data.cries[actionName].cries[v][criteria]) then
				message = message .. v .. " "
				table.insert(outputList, v)
			end
		else
			if(not BC_Data.cries[actionName].cries[v][criteria]) then
				message = message .. v .. " "
				table.insert(outputList, v)
			end
		end
	end
	BC_Core.Message("Pruned Cry List: " .. message, 4)
	return outputList
end

function BC_Core.GetCryList(actionName)
	BC_Core.Message("BC_Core.GetCryList(" .. tostring(actionName) .. ")", 2)
	local standardList = {}
	local standardListE = {}
	local chainList = {}
	local chainListE = {}
	local availableCries = {}
	for i, v in pairs(BC_Data.cries[actionName].cries) do
		if(v.chain) then
			if(v.chain[BC_Core.stored.lastCast]) then
				if(v.chain[BC_Core.stored.lastCast] == BC_Core.stored.lastCry) then
					if(BC_Conditions.Verify(v, actionName)==1) then
						if(v.exclusive) then
							table.insert(chainListE, i)
						else
							table.insert(chainList, i)
						end
					end
				end
			end
		elseif(not v.exclude) then
			if(BC_Conditions.Verify(v, actionName)==1) then
				if(v.exclusive) then
					table.insert(standardListE, i)
				else
					table.insert(standardList, i)
				end
			end
		end
	end
	local message = ""
	if(#(chainListE) > 0) then
		availableCries = chainListE
	elseif(#(chainList) > 0) then
		availableCries = chainList
	elseif(#(standardListE) > 0) then
		availableCries = standardListE
	elseif(#(standardList) > 0) then
		availableCries = standardList
	end
	for i=1, #(availableCries) do
		message = message .. availableCries[i] .. " "
	end
	BC_Core.Message("Original Cry List: " .. message, 4)
	return availableCries
end

function BC_Core.FormatCry(thisCry, actionName)
	BC_Core.Message("BC_Core.FormatCry(...)", 2)
	local newCry = {}
	if(thisCry.language) then
		newCry.language = GetLanguageByIndex(thisCry.language)
	else
		newCry.Language = GetLanguageByIndex(1)
	end
	if(thisCry.mode) then
		if(thisCry.mode == 1 ) then
			newCry.mode = "SAY";
		elseif(thisCry.mode == 2) then
			newCry.mode = "EMOTE";
		elseif(thisCry.mode == 3) then
			newCry.mode = "YELL";
		elseif(thisCry.mode == 4) then
			if(GetNumPartyMembers() > 0) then
				newCry.mode = "PARTY";
			else
				newCry.mode = "SAY";
			end
		else
			if(GetNumRaidMembers() > 0) then
				if(thisCry.mode == 6) then
					if(IsRaidLeader() or IsRaidOfficer()) then
						newCry.mode = "RAIDWARNING";
					else
						newCry.mode = "RAID";
					end
				else
					newCry.mode = "RAID";
				end			
			elseif(GetNumPartyMembers() > 0) then
				newCry.mode = "PARTY";
			else
				newCry.mode = "SAY";
			end
		end
	else
		newCry.mode = "SAY";
	end
	newCry.text = thisCry.text
	for i, v in ipairs(BC_Vars.Condition.Variables) do
		if(string.find(newCry.text, v)) then
			BC_Core.Message("found " .. v, 4)
			if(v == "$nm" and #(BC_Core.stored.target) > 0) then
				newCry.text = string.gsub(newCry.text, v, BC_Core.stored.target[actionName]);
			else
				func = getglobal(BC_Vars.Condition.Functions[i]);
				if(v == "$pn") then
					target = "pet";
				else
					target = "target";
				end
				_, retval = pcall(func,target);
				BC_Core.Message("Calling " .. BC_Vars.Condition.Functions[i] .. "(" .. target .. ") returned " .. tostring(retval), 3);
				if(retval) then
					if(v == "$po") then
						replaceText = BC_Vars.Pronouns.Object[retval];
					elseif(v == "$pp") then
						replaceText = BC_Vars.Pronouns.Possessive[retval];
					elseif(v == "$ps") then
						replaceText = BC_Vars.Pronouns.Subject[retval];
					elseif(v == "$ga") then
						replaceText = BC_Vars.Pronouns.Gender1[retval];
					elseif(v == "$gb") then
						replaceText = BC_Vars.Pronouns.Gender2[retval];
					else
						replaceText = retval;
					end
					newCry.text = gsub(newCry.text, v, replaceText);
				end
			end
		end
	end
	newCry.text = BC_Core.EventVariables_Replace(newCry.text);
	endNewLine = true;

	while(endNewLine) do
		if(strsub(thisCry.text,strlen(thisCry.text))=="\n") then
			BC_Core.Message("Pulling new line from end of string: '" .. thisCry.text .. "'", 3);
			newCry.text = strsub(thisCry.text,1,strlen(thisCry.text) - 1);
		else
			BC_Core.Message("No more new lines at end of string: '" .. thisCry.text .. "'", 3);
			endNewLine = false;
		end
	end
	if(thisCry.start) then
		newCry.start = true
	end
	return newCry;
end

function BC_Core.EventVariables_Replace(text)
	if(BC_Core.eventVars) then
		loc = string.find(text, "$e")
		while(loc) do
			index = tonumber(strsub(text,loc+2,loc+2))
			replaced = "$e" .. tostring(index)
			if(BC_Core.eventVars[index]) then
				text = gsub(text, replaced, BC_Core.eventVars[index])
				loc = strfind(text, "$e")
			else
				loc = nil
			end
		end
	end
	return text
end



--===================================================================================================================--

-- Execute Cry Functions

--===================================================================================================================--

function BC_Core.ExecuteCry(actionName, cryNumber)
	BC_Core.Message("BC_Core.ExecuteCry(" .. actionName .. ", " .. cryNumber .. ")", 2)
	BC_Core.Message(BC_Data.cries[actionName].cries[cryNumber].text, 4)
	BC_Core.SetCooldown(actionName)
	BC_Core.SendCry(BC_Data.cries[actionName].cries[cryNumber])
	BC_Core.stored.lastCast = actionName
	BC_Core.stored.lastCry = cryNumber
end

function BC_Core.ExecuteStoredStartCry(actionName)
	BC_Core.Message("BC_Core.ExecuteStoredStartCary(" .. actionName .. ")", 2)
	if(BC_Core.stored.cry[actionName]) then
		if(BC_Core.stored.cry[actionName].start) then
			BC_Core.Message(BC_Core.stored.cry[actionName].text, 4)
			BC_Core.SendCry(BC_Core.stored.cry[actionName])
			BC_Core.stored.cry[actionName] = nil
		end
	end
end

function BC_Core.ExecutePetCry(actionName)
	BC_Core.Message("BC_Core.ExecutePetCry(" .. actionName .. ")", 2)
	if(BC_Core.stored.cry[actionName]) then
		BC_Core.Message(BC_Core.stored.cry[actionName].text, 4)
		BC_Core.SendCry(BC_Core.stored.cry[actionName])
		BC_Core.stored.cry[actionName] = nil
	end
end

function BC_Core.ExecuteStoredCry(actionName)
	BC_Core.Message("BC_Core.ExecuteStoredCry(" .. actionName ..")", 2)
	if(BC_Core.stored.cry[actionName]) then
		BC_Core.Message(BC_Core.stored.cry[actionName].text, 4)
		BC_Core.SendCry(BC_Core.stored.cry[actionName])
		BC_Core.SetCooldown(actionName)
		BC_Core.stored.cry[actionName] = nil
		BC_Core.stored.target[actionName] = nil
	end
end

function BC_Core.ExecuteInterruptedCry(actionName)
	BC_Core.Message("BC_Core.ExecuteInterruptedCry(" .. actionName .. ")", 2)
	local cryNumber = nil
	local cryList = {}
	if(BC_Data.cries[actionName]) then
		chance = random(1,100)
		BC_Core.Message(chance .. " against frequency " .. BC_Data.cries[actionName].freq, 3)
		cryList = BC_Core.GetCryList(actionName)
		cryList = BC_Core.PruneCryList(actionName, cryList, "interrupt", true)
		if(#(cryList)>0) then
			if(chance <= BC_Data.cries[actionName].freq) then
				cryList = BC_Core.GetCryListMultiplied(actionName, cryList)
				cryNumber = cryList[random(1,#(cryList))]
			else
				cryList = BC_Core.GetCryListMultiplied(actionName, BC_Core.PruneCryList(actionName, cryList,"always",true))
			end
			if(#(cryList)>0) then
				cryNumber = cryList[random(1,#(cryList))]
				BC_Core.stored.cry[actionName] = BC_Core.FormatCry(BC_Data.cries[actionName].cries[cryNumber], actionName)
				BC_Core.stored.lastCast = actionName
				BC_Core.stored.lastCry = cryNumber
				BC_Core.ExecuteStoredCry(actionName)
				if(BC_Data.cries[actionName].cooldown > 0) then
					BC_Core.stored.cooldowns[actionName] = GetTime() + BC_Data.cries[actionName].cooldown
				end
			else
				BC_Core.stored.cry[actionName] = nil
				BC_Core.stored.lastCast = nil
				BC_Core.stored.lastCry = nil
			end
		end
	end
end

function BC_Core.SendCry(thisCry)
	BC_Core.Message("BC_Core.SendCry(...)",2);
	if(thisCry) then
		if(not thisCry.text) then
			BC_Core.Message("No cry text.", 3);
			return nil;
		end
		if(strfind(thisCry.text,"\n")) then
			multiLine = true;
			while(multiLine) do
				endLine = strfind(thisCry.text,"\n");
				thisLine = strsub(thisCry.text,1, endLine-1);
				BC_Core.SendChatMessage(thisLine, thisCry.mode, thisCry.language);
				thisCry.text = strsub(thisCry.text,endLine+1,strlen(thisCry.text));
				BC_Core.Message("thisCry.text: |" .. thisCry.text .. "|", 3);
				if(not strfind(thisCry.text,"\n")) then multiLine = false; end
			end
			BC_Core.SendChatMessage(thisCry.text, thisCry.mode, thisCry.language);
		else
			BC_Core.SendChatMessage(thisCry.text, thisCry.mode, thisCry.language);
		end
	else
		BC_Core.Message("No cry inputed.", 3);
	end
end

function BC_Core.SendChatMessage(text, mode, language)
	BC_Core.Message("BC_Core.SendChatMessage(...)", 2);
	if(BC_Vars.DebugLevel > 0) then
		BC_Core.Message("[" .. tostring(mode) .. "] (" .. tostring(language) .. ") " .. tostring(text), 1);
	else
		if(strsub(text,1,1)=="/") then
			BC_Core.Message("Slash command found: " .. text, 3);
			BC_CoreFrame_ExecBox:SetText(text);
			ChatEdit_SendText(BC_CoreFrame_ExecBox);
			BC_CoreFrame_ExecBox:SetText("");
		else		
			SendChatMessage(text, mode, language);
		end
	end
end

function BC_Core.SetCooldown(actionName)
	BC_Core.Message("BC_Core.SetCooldown(" .. actionName .. ")", 2);
	if(tonumber(BC_Data.cries[actionName].cooldown) > 0) then
		BC_Core.Message(actionName .. " is now on cooldown.", 3);
		BC_Core.stored.cooldowns[actionName] = GetTime() + BC_Data.cries[actionName].cooldown
	end
end

function BC_Core.UnsetCooldown(actionName)
	BC_Core.Message("BC_Core.UnsetCooldown(" .. actionName .. ")", 2);
	BC_Core.stored.cooldowns[actionName] = nil
end

function BC_Core.OnCooldown(actionName)
	BC_Core.Message("BC_Core.OnCooldown(" .. actionName .. ")", 2);
	if(BC_Core.stored.cooldowns[actionName]) then
		if(GetTime() < BC_Core.stored.cooldowns[actionName]) then
			return true
		end
	end
	return false
end

--===================================================================================================================--

-- Hook Functions

--===================================================================================================================--

function BC_Core.CastPetAction(index, unit)
	BC_Core.Message("BC_Core.CastPetAction(" .. tostring(index) .. ", " .. tostring(unit) .. ")", 2)
	actionName, _, _, _, _, _, _ = GetPetActionInfo(index)
	BC_Core.Message(actionName, 3)
	if(actionName == "PET_ACTION_ATTACK") then 
		BC_Core.DetermineCry(BC_Vars.PetActions.Attack)
		BC_Core.ExecuteStoredCry(BC_Vars.PetActions.Attack)
	elseif(actionName == "PET_ACTION_FOLLOW") then 
		BC_Core.DetermineCry(BC_Vars.PetActions.Follow)
		BC_Core.ExecuteStoredCry(BC_Vars.PetActions.Follow)
	elseif(actionName == "PET_ACTION_WAIT") then 
		BC_Core.DetermineCry(BC_Vars.PetActions.Wait)
		BC_Core.ExecuteStoredCry(BC_Vars.PetActions.Wait)
	elseif(actionName == "PET_MODE_AGGRESSIVE") then 
		BC_Core.DetermineCry(BC_Vars.PetActions.Aggressive)
		BC_Core.ExecuteStoredCry(BC_Vars.PetActions.Aggressive)
	elseif(actionName == "PET_MODE_DEFENSIVE") then 
		BC_Core.DetermineCry(BC_Vars.PetActions.Defensive)
		BC_Core.ExecuteStoredCry(BC_Vars.PetActions.Defensive)
	elseif(actionName == "PET_MODE_PASSIVE") then 
		BC_Core.DetermineCry(BC_Vars.PetActions.Passive)
		BC_Core.ExecuteStoredCry(BC_Vars.PetActions.Passive)
	else
		BC_Core.DetermineCry("Pet " .. actionName)
		BC_Core.ExecuteStoredStartCry("Pet " .. actionName)
	end
end