BC_Events = { 
	registered = {},
}

--===================================================================================================================--

-- Initialize/Registration Functions

--===================================================================================================================--
function BC_Events.Register(eventName)
	BC_Core.Message("BC_EventCry_Register(" .. eventName .. ")", 2)
	if(BC_Events.registered[eventName]) then
		BC_Core.Message(eventName .. " already registered.", 3)
		BC_Events.registered[eventName] = BC_Events.registered[eventName] + 1
	else
		BC_Core.Message("Registering event " .. eventName, 3)
		BC_Events.registered[eventName] = 1
		getglobal("BC_CoreFrame"):RegisterEvent(eventName)
	end
end

function BC_Events.Unregister(eventName)
	BC_Core.Message("BC_EventCry_Unegister(" .. eventName .. ")",2)
	if(BC_Events.registered[eventName]) then
		BC_Events.registered[eventName] = BC_Events.registered[eventName] - 1
		if(BC_Events.registered[eventName] == 0) then
			BC_Core.Message("Unregistering event " .. eventName, 2)
			getglobal("BC_CoreFrame"):UnregisterEvent(eventName)
			BC_Events.registered[eventName] = nil
		end
	else
		BC_Core.Message(eventName .. " already unregistered.",3)
	end
end

function BC_Events.Initialize()
	BC_Core.Message("BC_Events.Initialize()", 2)
	local savedEvents = 0
	if(BC_Data.events) then
		savedEvents = #(BC_Data.events)
	end
	if(savedEvents == 0) then
		BC_Core.Message("No Events Saved.", 3)
		BC_Data.events = {
			["TRAINER_SHOW"] = {
				[1] = {	["name"] = "Visit Trainer", },
			},
			["DUEL_FINISHED"] = {
				[1] = {	["name"] = "Duel Finish", },
			},
			["DUEL_REQUESTED"] = {
				[1] = {	["name"] = "Duel Requested", },
			},
			["PLAYER_LEVEL_UP"] = {
				[1] = {	["name"] = "You Level Up", },
			},
			["PLAYER_CONTROL_LOST"] = {
				[1] = {	["name"] = "You Lose Control", },
			},
			["TRADE_SHOW"] = {
				[1] = {	["name"] = "Trade Window Opened", },
			},
			["PLAYER_REGEN_ENABLED"] = {
				[1] = {	["name"] = "Leave Combat", },
			},
			["PLAYER_REGEN_DISABLED"] = {
				[1] = {	["name"] = "Enter Combat", },
			},
			["CHAT_MSG_COMBAT_HOSTILE_DEATH"] = {
				[1] = {	["name"] = "You Slay Hostile",
					["args"] = {
						[1] = "You have slain %s!",
					},
				},
			},
			["CHAT_MSG_SPELL_SELF_DAMAGE"] = {
				[1] = {	["name"] = "You Crit (Special)",
					["args"] = {
						[1] = "Your %s crits %s for %d %s damage.",
					},
				},
				[2] = {	["name"] = "You Hit (Special)",
					["args"] = {
						[1] = "Your %s hits %s for %d %s damage.",
					},
				},
			},
			["CHAT_MSG_COMBAT_SELF_HITS"] = {
				[1] = {	["name"] = "You Crit (Physical)",
					["args"] = {
						[1] = "You crit %s for %d.",
					},
				},
			},
			["CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS"] = {
				[1] = {	["name"] = "You Get Crit (Physical)",
					["args"] = {
						[1] = "%s crits you for %d.",
					},
				},
			},
			["COMBAT_TEXT_UPDATE"] = {
				[1] = {	["name"] = "Buff Gained",
					["args"] = {
						[1] = "AURA_START",
					},
				},
				[2] = {	["name"] = "Buff Lost",
					["args"] = {
						[1] = "AURA_END",
					},
				},
				[3] = {	["name"] = "Debuff Gained",
					["args"] = {
						[1] = "AURA_START_HARMFUL",
					},
				},
				[4] = {	["name"] = "Debuff Lost",
					["args"] = {
						[1] = "AURA_END_HARMFUL",
					},
				},
			},
		}
	end
end

function BC_Events.RegisterEvents()
	BC_Core.Message("BC_Events.RegisterEvents()", 2)
	for k, v in pairs(BC_Data.events) do
		BC_Core.Message("Examining " .. k, 3)		
		for i, w in ipairs(v) do
			BC_Core.Message("Examining [" .. i .. "] (" .. w.name .. ") of " .. k, 4)	
			if(BC_Data.cries[w.name]) then
				BC_Core.Message("There is an action saved by the name of " .. w.name, 5)
				if(#(BC_Data.cries[w.name].cries) > 0) then
					BC_Events.Register(k)
				end
			end
		end
	end
end

--===================================================================================================================--

-- Event Execution Functions

--===================================================================================================================--
function BC_Events.Process(event)
	BC_Core.Message("BC_Events.Process(" .. event .. ")", 2)
	if(BC_Data.events[event]) then
		BC_Core.Message("Found event watches for " .. event, 3)
		for i, w in ipairs(BC_Data.events[event]) do
			if(BC_Data.cries[w.name]) then
				if(w.args) then
					found = true
					for k, v in ipairs(w.args) do
						if(BC_Events.CompareMessages(v, getglobal("arg" .. k)) == false) then
							found = false
						end
					end
					if(found) then
						BC_Core.Message("Compare successful with " .. w.name, 4)
						BC_Core.DetermineCry(w.name)
						BC_Core.ExecuteStoredCry(w.name)
					end
				else
					BC_Core.DetermineCry(w.name)
					BC_Core.ExecuteStoredCry(w.name)
				end
			end
		end
	end
end

function BC_Events.ReturnVariable(index)
	BC_Core.Message("BC_Events.ReturnVariable(" .. index .. ")", 2)
	return BC_Core.stored.eventVars[index] or nil
end

function BC_Events.ReplaceVariables(text)
	BC_Core.Message("BC_Events.ReplaceVariables(" .. text .. ")", 2)
	loc = string.find(text, "$e")
	while(loc) do
		index = tonumber(string.sub(text,loc+2,loc+2))
		replaced = "$e" .. tostring(index)
		if(BC_Core.eventVars[index]) then
			BC_Core.Message("Replacing " .. replaced .. " with " .. BC_Core.eventVars[index], 3)
			text = string.gsub(text, replaced, BC_Core.eventVars[index])
			loc = string.find(text, "$e")
		else
			loc = nil
		end
	end
	return text
end

function BC_Events.CompareMessages(search, eventMessage)
	BC_Core.Message("BC_Events.CompareMessages(" .. tostring(search) .. ", " .. tostring(eventMessage) .. ")", 2)
	if(search == eventMessage) then
		return true
	end
	if(not eventMessage) then
		return false
	end
	-- The following lines needed for french translation. Unsure if still necessary
		search = string.gsub(search, "%(", "")
		search = string.gsub(search, "%)", "")
		eventMessage = string.gsub(eventMessage, "%(", "")
		eventMessage = string.gsub(eventMessage, "%)", "")
	pieces = {}
	locStart = 1
	locEnd, _ = string.find(search, "%%")
	while locEnd do
		table.insert(pieces, string.sub(search, locStart, locEnd-1))
		locStart = locEnd + 2
		locEnd = string.find(search, "%%", locStart )
	end
	table.insert(pieces, string.sub(search, locStart))
	for k, v in pairs(pieces) do
		BC_Core.Message("<" .. v .. ">", 4)
		if string.find(eventMessage, v) == nil then
			return false
		end
	end
	BC_Core.Message("Message found. " .. eventMessage, 3)
	BC_Core.eventVars = {}
	for i=1, table.getn(pieces) do
		locStart = string.find(eventMessage, pieces[i],1,true)
		BC_Core.Message("Message is: <" .. eventMessage .. ">, piece is: <" .. pieces[i] .. "> found at " .. tostring(locStart), 3)
		if(locStart == 1) then
			BC_Core.Message("Stripped out: <" .. pieces[i] .. ">", 4)
			eventMessage = string.gsub(eventMessage, pieces[i], "", 1)
		else
			BC_Core.Message("Stripped out: <" .. strsub(eventMessage, 1, locStart+strlen(pieces[i])-1) .. ">", 4)
			table.insert(BC_Core.eventVars, string.sub(eventMessage, 1, locStart-1))
			eventMessage = string.gsub(eventMessage, strsub(eventMessage, 1, locStart-1), "", 1)
			eventMessage = string.gsub(eventMessage, pieces[i], "", 1)
		end
	end
	return true
end