BC_UI.SubFrame = {}
BC_UI.SubFrame.__index = BC_UI.SubFrame

function BC_UI.SubFrame:new(name, label, width, height)
	self = {}
	name = this:GetName() .. "_" .. name .. "Frame"
	self.frame = CreateFrame("Frame", name, this, "BC_UI.SubFrameTemplate")
	self.frame:SetWidth(width)
	self.frame:SetHeight(height)
	getglobal(name .. "_TitleText"):SetText(label)
	return self
end

BC_UI.TabButton = {}
BC_UI.TabButton.__index = BC_UI.TabButton

function BC_UI.TabButton:new(name, label, id)
	self = {}
	name = this:GetName() .. "Tab" .. id
	self.frame = CreateFrame("CheckButton", name, this, "BC_BottomTabButtonTemplate")
	self.frame:SetID(id)
	self.frame:SetText(label)
	PanelTemplates_TabResize(10, self.frame)
	getglobal(name .."HighlightTexture"):SetWidth(self.frame:GetTextWidth() + 30)
	self.frame:SetScript("OnClick",	BC_Editor.Tab_OnClick);
	return self
end

BC_UI.DropDown = {}
BC_UI.DropDown.__index = BC_UI.DropDown

function BC_UI.DropDown:new(name, label, width, func)
	self = {}
	name = this:GetName() .. "_" .. name .. "DropDown"
	self.frame = CreateFrame("Frame", name, this, "UIDropDownMenuTemplate")
	self.label = CreateFrame("Frame", name .. "Label", self.frame, "BC_LabelTemplate")
	self.label:SetPoint("BOTTOMLEFT", name, "TOPLEFT", 20, 0)
	self.label:Show()
	getglobal(name .. "LabelText"):SetText(label)
	UIDropDownMenu_JustifyText("LEFT", self.frame)
	UIDropDownMenu_SetWidth(width, self.frame)
	UIDropDownMenu_Initialize(self.frame, func)
	return self
end

BC_UI.Button = {}
BC_UI.Button.__index = BC_UI.Button

function BC_UI.Button:new(name, text, func)
	self = {}
	name = this:GetName() .. "_" .. name .. "Button"
	self.frame = CreateFrame("Button", name, this, "UIPanelButtonTemplate")
	self.frame:SetScript("OnClick", func)
	getglobal(name .. "Text"):SetText(text)
	self.frame:SetHeight(24)
	return self
end

BC_UI.CheckButton = {}
BC_UI.CheckButton.__index = BC_UI.CheckButton

function BC_UI.CheckButton:new(name, value, text)
	self = {}
	self.value = value
	name = this:GetName() .. "_" .. name .. "CheckButton"
	self.frame = CreateFrame("CheckButton", name, this, "BC_UI.CheckButtonTemplate")
	getglobal(name .. "Text"):SetText(text)
	self.frame:SetChecked(value)
	return self
end

BC_UI.EditBox = {}
BC_UI.EditBox.__index = BC_UI.EditBox

function BC_UI.EditBox:new(name, label, default, size)
	self = {}
	name = this:GetName() .. "_" .. name .. "EditBox"
	self.frame = CreateFrame("EditBox", name, this, "BC_UI.EditBoxTemplate")
	self.frame:SetText(default)
	self.label = CreateFrame("Frame", name .. "Label", self.frame, "BC_LabelTemplate")
	self.label:SetPoint("BOTTOMLEFT", name, "TOPLEFT", 0, -3)
	self.label:Show()
	getglobal(name .. "LabelText"):SetText(label)
	getglobal(name .. "_TextureCenter"):SetWidth(size-10)
	self.frame:SetWidth(size)
	self.frame:SetHeight(32)
	return self
end

BC_UI.EditBoxScroll = {}
BC_UI.EditBoxScroll.__index = BC_UI.EditBoxScroll

function BC_UI.EditBoxScroll:new(name, label, default, width, height)
	self = {}
	name = this:GetName() .. "_" .. name .. "EditBoxScroll"
	self.frame = CreateFrame("Frame", name, this, "BC_UI.EditBoxScrollFrameTemplate")
	getglobal(name .. "ScrollChildEditBox"):SetText(default)
	self.label = CreateFrame("Frame", name .. "Label", self.frame, "BC_LabelTemplate")
	self.label:SetPoint("BOTTOMLEFT", name, "TOPLEFT", 5, 0)
	self.label:Show()
	getglobal(name .. "LabelText"):SetText(label)
	self.frame:SetWidth(width)
	self.frame:SetHeight(height)
	getglobal(name .. "Scroll"):SetWidth(width - 23)
	getglobal(name .. "Scroll"):SetHeight(height - 12)
	getglobal(name .. "ScrollChild"):SetWidth(width - 10)
	getglobal(name .. "ScrollChild"):SetHeight(height - 10)
	getglobal(name .. "ScrollChildEditBox"):SetTextInsets(0,27,4,0)
	getglobal(name .. "ScrollChildEditBox"):SetWidth(width - 5)
	getglobal(name .. "ScrollChildEditBox"):SetHeight(height)
	return self
end

BC_UI.HSlider = {}
BC_UI.HSlider.__index = BC_UI.HSlider

function BC_UI.HSlider:new(name, label, minVal, maxVal, stepVal, startVal)
	self = {}
	self.value = value
	name = this:GetName() .. "_" .. name .. "Slider"
	self.frame = CreateFrame("Slider", name, this, "BC_UI.HSliderTemplate")
	self.frame:SetMinMaxValues(minVal, maxVal)
	self.frame:SetValueStep(stepVal)
	self.frame:SetValue(startVal)
	getglobal(name .. "Text"):SetText(label)
	getglobal(name .. "Low"):SetText(minVal)
	getglobal(name .. "High"):SetText(maxVal)
	getglobal(name .. "TextUpdate"):SetText(startVal)
	self.frame:SetWidth(170)
	self.frame:SetHeight(17)
	return self
end

BC_UI.ScrollPanel = {}
BC_UI.ScrollPanel.__index = BC_UI.ScrollPanel

function BC_UI.ScrollPanel:new(name, size, list, func)
	name = this:GetName() .. "_" .. name
	self = {}
	self.size = size
	self.list = list
	self.func = func
	self.selected = 0
	self.frame = CreateFrame("Frame", name, this, "BC_ScrollListTemplate")
	self.frame:SetWidth(190)
	self.frame:SetHeight(12)
	for i=1, size do
		BC_UI.ScrollList_AddButton(self.frame, self.list[i])
	end
	return self
end

function BC_UI.ScrollList_OnUpdate()
--	BC_Core.Message("BC_UI.ScrollList_OnUpdate(" .. this:GetName() .. ")",3);
	local thisScroll = string.sub(this:GetName(), string.len(this:GetParent():GetParent():GetName()) + 2)
	local tempList = {}
	local scrollSize = 0
	local selected = 0
	if(BC_EditorUI[thisScroll]) then
		tempList = BC_EditorUI[thisScroll].list
		scrollSize = BC_EditorUI[thisScroll].size
		selected = BC_EditorUI[thisScroll].selected
	end
	BC_UI.ScrollList_UpdateList(this, scrollSize, tempList, #(tempList), selected)
end

function BC_UI.ScrollList_UpdateList(frame, scrollSize, list, listSize, selected)
--	BC_Core.Message("BC_UI.ScrollList_UpdateList(" .. frame:GetName() .. ", " .. scrollSize .. ", list[...], " .. listSize .. ", " .. selected, 3);
	FauxScrollFrame_Update(frame, listSize, scrollSize, 16)
	local line
	local offset = FauxScrollFrame_GetOffset(frame);
	for line = 1, scrollSize do
		if(line + offset <= listSize) then
			getglobal(frame:GetName() .. "ListItem" .. line):SetText(list[line + offset]);
			getglobal(frame:GetName() .. "ListItem" .. line):Show();
		else
			getglobal(frame:GetName() .. "ListItem" .. line):Hide();
		end
	end
	highlight = getglobal(frame:GetParent():GetName() .. "Highlight")
	if(selected > offset and selected <= (offset + scrollSize)) then
		selectedButton = getglobal(frame:GetName() .. "ListItem" .. selected - offset)
		highlight:SetPoint("TOPLEFT", selectedButton)
		highlight:Show()
	else
		highlight:Hide()
	end
end

function BC_UI.ScrollList_AddButton(frame, text)
	local item = frame:GetNumChildren() - 1
	local buttonName = frame:GetName() .. "ScrollListItem" .. item
	local button = CreateFrame("Button", buttonName, frame, "BC_ListButtonTemplate")
	button:SetID(item)
	button:SetText(text)
	button:Show()
	local frameHeight = frame:GetHeight()
	frame:SetHeight(frameHeight + 16)
	local scroll = getglobal(frame:GetName() .. "Scroll")
	scroll:SetHeight(scroll:GetHeight() + 16)
	if(item == 1) then
		button:SetPoint("TOPLEFT", scroll)
		button:SetPoint("TOPRIGHT", scroll, "TOPRIGHT", 0, -20)
	else
		item = item - 1
		local otherButton = frame:GetName() .. "ScrollListItem" .. item
		button:SetPoint("TOPLEFT", otherButton, "BOTTOMLEFT")
		button:SetPoint("TOPRIGHT", otherButton, "BOTTOMRIGHT")
	end
end

function BC_UI.ScrollList_SetWidth(panel, width)
	BC_Core.Message("BC_UI.ScrollList_SetWidth(" .. panel.frame:GetName() .. ", " .. width .. ")", 2)
	panel.frame:SetWidth(width)
	getglobal(panel.frame:GetName() .. "Scroll"):SetWidth(width)
	getglobal(panel.frame:GetName() .. "HighlightTexture"):SetWidth(width)
end

function BC_UI.ScrollList_Select(panel, selected)
	BC_Core.Message("BC_UI.ScrollList_Select(" .. panel.frame:GetName() .. ", " .. selected .. ")", 2)
	panel.selected = selected
	offset = FauxScrollFrame_GetOffset(getglobal(panel.frame:GetName() .. "Scroll"))
	viewMin = offset + 1
	viewMax = viewMin + panel.size - 1
	if(selected == 0) then
		getglobal(frame:GetParent():GetName() .. "Highlight"):Hide()
	elseif(selected >= viewMin) then
		if(selected > viewMax) then
			offset = selected - panel.size + 1
			FauxScrollFrame_SetOffset(getglobal(panel.frame:GetName() .. "Scroll"), offset)
		end
	else
		offset = selected - panel.size + 1
		if(offset < 0) then
			offset = 0
		end
		FauxScrollFrame_SetOffset(getglobal(panel.frame:GetName() .. "Scroll"), offset)
	end
	BC_UI.ScrollList_UpdateList(getglobal(panel.frame:GetName() .. "Scroll"), panel.size, panel.list, #(panel.list), selected)
	getglobal(panel.frame:GetName() .. "ScrollListItem" .. (selected - offset)):Click()
end

function BC_UI.ScrollListButton_OnClick()
	local thisScroll = string.sub(this:GetName(), string.len(this:GetParent():GetParent():GetName()) + 2, string.find(this:GetName(), "ListItem") - 1)
	local item = this:GetID()
	local offset = FauxScrollFrame_GetOffset(getglobal(this:GetParent():GetName() .. "Scroll"))
	local func
	if(BC_EditorUI[thisScroll]) then
		BC_EditorUI[thisScroll].selected = item + offset
		pcall(BC_EditorUI[thisScroll].func, item + offset)
	end
	highlight = getglobal(this:GetParent():GetName() .. "Highlight")
	highlight:SetPoint("TOPLEFT", this)
	highlight:Show()
end
